function [z,M1,M0,alpha,bp,bn,L]=SBM_GIBBS(A,K,opts)

% Usgage
% [z,E_eta,E_log_eta]=SBM_GIBBS(A,noc)
%
% Model
%   \pi~Dirichtlet(alpha/K 1)
%   z_i~Cat(\pi)
%   \eta(m)_lh~beta(bp,bn)
%   A(m)_ij~Bernoulli(eta(m)_(z_i,z_j))
%
%  \pi and \eta analytically marginalized
%
%  Joint Distribution:
%
%  
% Input:
% A     cell array of N x N undirected connectomes
% K     number of components
% opts. struct with fields
%       alpha (default 1)
%       bp (default 1)
%       bn (default 1)
%       T (default 100 sampling iterations) 
%       sampleZ (default true)
%       sampleHyper (default ???)
% Output:
% z         last sample partition structure
% M1        Links between parcels of last sample
% M0        non-links between parcels of last sample
% alpha     last sample alpha value
% bp        last sample bp value
% bn        last sample bn value
% L         Joint distribution across samples
%
% GIBBS sampler of z described in 
% Schmidt, Mikkel N., and Morten Morup. "Nonparametric Bayesian modeling of complex networks: An introduction." IEEE Signal Processing Magazine 30.3 (2013): 110-128.
% here adapted to the finite SBM instead of IRM
%
% Written by Morten M?rup

if ~isfield(opts,'bn'),     bn = 1; else,     bn=opts.bn; end
if ~isfield(opts,'bp'),     bp = 1; else,     bp=opts.bp; end
if ~isfield(opts,'alpha'),  alpha = 1; else,  alpha=opts.alpha; end
if ~isfield(opts,'T'),      T = 100; else,    T=opts.T; end
if ~isfield(opts,'debug'),  debug=false; else, debug=opts.debug; end
if ~isfield(opts,'sampleHyper'), sampleHyper=true; else, sampleHyper=opts.sampleHyper; end
if ~isfield(opts,'sampleZ'), sampleZ=true; else, sampleZ=opts.sampleZ; end

if ~iscell(A)
    AA=A;
    clear A;
    A{1} = AA;
end

L=nan(1,T);

% randomly initialize z
N=size(A{1},1);
z = ceil(K*rand(1,N));
Z = sparse(z,1:N,ones(N,1),K,N);

for m=1:length(A)
    % ensure A is symmetric undirected with no self links
    A{m} = triu(A{m},1);
    A{m} = A{m}+A{m}';  
end

% Calculate initial between parcels link and non-link counts
[M1, M0, Nk]=getLinkStats(A,Z);

for t = 1:T % For each Gibbs sweep
    disp(['working on iteration ' num2str(t)])
    
    if sampleZ
        for n = randperm(N) % For each node in the graph                 

            % remove node n from all statistics
             Nk(z(n))=Nk(z(n))-1;             
             for m=1:length(A)
                 r1(:,m)=full(Z*A{m}(:,n));
                 M1(:,z(n),m)=M1(:,z(n),m)-r1(:,m);
                 M1(z(n),:,m)=M1(z(n),:,m)-r1(:,m)';
                 M1(z(n),z(n),m)=M1(z(n),z(n),m)+r1(z(n),m);

                 r0(:,m)=Nk-r1(:,m);
                 M0(:,z(n),m)=M0(:,z(n),m)-r0(:,m);
                 M0(z(n),:,m)=M0(z(n),:,m)-r0(:,m)';
                 M0(z(n),z(n),m)=M0(z(n),z(n),m)+r0(z(n),m);
             end              
             R1=repmat(permute(r1,[1 3 2]),[1,K,1]);
             R0=repmat(permute(r0,[1,3,2]),[1,K,1]);         
             logP = sum(sum(betaln(M1+R1+bp,M0+R0+bn)-betaln(M1+bp,M0+bn),1),3) + (gammaln(Nk'+1+alpha/K)-gammaln(Nk'+alpha/K)); 
             P = exp(logP-max(logP)); % Convert from log probability

             if debug
                for k=1:K
                    Zt=Z;
                    Zt(:,n)=0;
                    Zt(k,n)=1;
                    [M1t,M0t,Nkt]=getLinkStats(A,Zt); 
                    logPP(k) = log_marg_lik(M1t,M0t,bp,bn)+log_prior_z(Nkt,alpha);
                end

                PP = exp(logPP-max(logPP)); % Convert from log probability
                PP=PP/sum(PP);            
                if norm(P/sum(P)-PP)>1e-10
                    disp('sampling distribution wrong');
                    keyboard
                else
                    disp('sampling distribution correct');                
                end
             end

             z(n) = find(rand<cumsum(P)/sum(P),1); % Random component according to P
             Z = sparse(z,1:N,ones(1,N),K,N);

             % update statistics         
             for m=1:length(A)             
                 M1(:,z(n),m)=M1(:,z(n),m)+r1(:,m);
                 M1(z(n),:,m)=M1(z(n),:,m)+r1(:,m)';
                 M1(z(n),z(n),m)=M1(z(n),z(n),m)-r1(z(n),m);

                 M0(:,z(n),m)=M0(:,z(n),m)+r0(:,m);
                 M0(z(n),:,m)=M0(z(n),:,m)+r0(:,m)';
                 M0(z(n),z(n),m)=M0(z(n),z(n),m)-r0(z(n),m);
             end
             Nk(z(n))=Nk(z(n))+1;             

             if debug
                 [M1t,M0t,Nkt]=getLinkStats(A,Z); % Debug
                 if norm(M1(:)-M1t(:))>0
                     disp('error in link counts')
                 elseif norm(M0(:)-M0t(:))>0
                     disp('error in non-link counts')
                 elseif norm(Nk-Nkt)>0
                     disp('error in cluster size')
                 else
                     disp('Statistics fine');
                 end
             end                                 
        end
    end
    
    if sampleHyper        
        %sample bp
        bp=sample_bp(M1,M0,bp,bn);
    
        % sample bn
        bn=sample_bn(M1,M0,bp,bn);
    
        % Sample alpha
        alpha=sample_alpha(Nk,alpha);   
    end
    
    % Evaluate likelihood
    L(t) = log_marg_lik(M1,M0,bp,bn)+log_prior_z(Nk,alpha);
end

%---------------------------------
function logP=log_marg_lik(M1,M0,bp,bn)

logP=0;
for m=1:size(M1,3)
    logP=logP+sum(sum(triu(betaln(M1(:,:,m)+bp,M0(:,:,m)+bn)-betaln(bp,bn))));
end

%----------------------------------
function logPz=log_prior_z(Nk,alpha)

logPz=gammaln(alpha)-gammaln(alpha+sum(Nk))+sum(gammaln(Nk+alpha/length(Nk))-gammaln(alpha/length(Nk)));

%----------------------------------
function [M1,M0, Nk]=getLinkStats(A,Z)
    
K=size(Z,1);
Nk = full(sum(Z,2));
M1 = nan(K,K,length(A));
M0 = nan(K,K,length(A));
for m=1:length(A)
    M1(:,:,m) = Z*A{m}*Z';
    M1(:,:,m) = M1(:,:,m)-0.5*diag(diag(M1(:,:,m)));    
    M0(:,:,m) = Nk*Nk'-diag(Nk);
    M0(:,:,m) = M0(:,:,m)-0.5*diag(diag(M0(:,:,m)))-M1(:,:,m);    
end

%----------------------------
function alpha=sample_alpha(Nk,alpha)
% assumes uniform prior and symmetric normal(0,1) proposal
logP=log_prior_z(Nk,alpha);
for proposals=1:1000
    alpha_new=alpha+randn;
    if alpha_new>0 % reject sample if negative
        logP_new=log_prior_z(Nk,alpha_new);    
        if exp(logP_new-logP)>rand
            alpha=alpha_new;
            logP=logP_new;
        end    
    end
end


%----------------------------
function bp=sample_bp(M1,M0,bp,bn)
% assumes uniform prior and symmetric normal(0,1) proposal
logP=log_marg_lik(M1,M0,bp,bn);
for proposals=1:1000
    bp_new=bp+randn;
    if bp_new>0 % reject sample if negative
        logP_new=log_marg_lik(M1,M0,bp_new,bn);    
        if exp(logP_new-logP)>rand
            bp=bp_new;
            logP=logP_new;
        end    
    end
end

%----------------------------
function bn=sample_bn(M1,M0,bp,bn)
% assumes uniform prior and symmetric normal(0,1) proposal
logP=log_marg_lik(M1,M0,bp,bn);
for proposals=1:1000
    bn_new=bn+randn;
    if bn_new>0 % reject sample if negative
        logP_new=log_marg_lik(M1,M0,bp,bn_new);    
        if exp(logP_new-logP)>rand
            bn=bn_new;
            logP=logP_new;
        end    
    end
end
