% Script that generates synthetic data example based on the stochastic block model (SBM) and
% analyses each modality separately and jointly using the described SBM
% modeling approach.

clear all;
K=10; % Number of clusters
N=1000; % Number of nodes
Ngraphs=5; % Number of graphs pr. modality
bias1=0.1; % degree of bias from true underlying parcellation in modality 1
bias2=0.1; % degree of bias from true underlying parcellation in modality 2
noise_bias1=0.25; % Degree of bias in terms of noisy connectivity in modality 1
noise_bias2=0.25; % Degree of bias in terms of noisy connectivity in modality 2

% Paramters for SBM
bp=0.1; 
bn=9.9;
alpha=5*K;

%% Generate synthetic datasets
% sample from dirichlet distribution the partitions
z=nan(N,1);
pi_K = gamrnd(alpha/K*ones(K,1),ones(K,1));
pi_K=pi_K/sum(pi_K);
for n=1:N    
    z(n) = find(rand<cumsum(pi_K),1); 
end
z=sort(z,'ascend'); % organize generated parcellation into blocks
Z=sparse(z,1:N,ones(1,N),K,N);

% generate partitions in two modalities potentially with bias from ground
% truth Z
Z1=Z;
Z2=Z;
idx1=randperm(N);
idx1=idx1(1:ceil(bias1*N));
Z1(:,idx1)=Z1(:,idx1(randperm(length(idx1))));   
idx2=randperm(N);
idx2=idx2(1:ceil(bias2*N));
Z2(:,idx2)=Z2(:,idx2(randperm(length(idx2))));   

% Generate networks
figure; 
eta=nan(K,K,2);
for m=1:2
    if m==1
        Zt=Z1;
    else
        Zt=Z2;
    end    
    if m==1
        eta(:,:,m)=(1-noise_bias1)*betarnd(bp*ones(K),bn*ones(K));
        Am=triu((noise_bias1*bp/(bp+bn)*ones(N))>rand(N),1);
    else
        eta(:,:,m)=(1-noise_bias2)*betarnd(bp*ones(K),bn*ones(K));
        Am=triu((noise_bias2*bp/(bp+bn)*ones(N))>rand(N),1);
    end
    for g=1:Ngraphs
        A{m,g}=sparse((triu(Zt'*eta(:,:,m)*Zt>rand(N,N),1)+Am)>0);
        
        subplot(2,Ngraphs,g+(m-1)*Ngraphs);
        spy(A{m,g}+A{m,g}');
        title(['Network ' num2str(g)]);        
        if g==1
            ylabel(['Modality ' num2str(m)]);
        end
    end
end


%% Run SBM analysis to learn parcellations for each modality and jointly (indicated by m=3)
opts.bp=1;
opts.bn=1;
opts.alpha=1;
opts.T=100;
for m=1:3
    for g=1:Ngraphs
        if m<=2
            [z_e{m,g},~,~,~,bp_e(m,g),bn_e(m,g),L{m,g}]=SBM_GIBBS(A(m,g),K,opts);
        else % Joint model of both graphs           
            [z_e{m,g},~,~,~,bp_e(m,g),bn_e(m,g),L{m,g}]=SBM_GIBBS(A(1:2,g),K,opts);
        end
    end
end


%% Predictive assessment
for m=1:3 % Loop across parcellations
    for mm=1:2 % Loop across modalities evaluating the parcellation
        t=0;
        for g=1:Ngraphs % Loop across training graphs
            for gg=[1:g-1 g+1:Ngraphs] % Loop across test graphs
                t=t+1;
                [~, predLL(m,mm,t), AUC(m,mm,t), ~, ~]=evaluateParcellation(z_e{m,g}',A{mm,g},A{mm,gg},[bp_e(m,g) bn_e(m,g)]);
            end
        end
    end    
end

figure;
subplot(2,2,1)
boxplot(squeeze(predLL(:,1,:))','labels',{'Mod 1',' Mod 2', 'Joint'})
title('predicting on modality 1')
ylabel('PredLL')

subplot(2,2,2)
boxplot(squeeze(AUC(:,1,:))','labels',{'Mod 1',' Mod 2', 'Joint'})
title('predicting on modality 1')
ylabel('AUC')

subplot(2,2,3)
boxplot(squeeze(predLL(:,2,:))','labels',{'Mod 1',' Mod 2', 'Joint'})
title('predicting on modality 2')
ylabel('PredLL')

subplot(2,2,4)
boxplot(squeeze(AUC(:,2,:))','labels',{'Mod 1',' Mod 2', 'Joint'})
title('predicting on modality 2')
ylabel('AUC')

%% NMI evaluation to ground truth Z
for m=1:3
    for g=1:Ngraphs
        Z_e=sparse(z_e{m,g},1:N,ones(1,N),K,N);
        NMI(m,g)=calcNMI(Z_e,Z);
    end
end
figure;
boxplot(NMI','labels',{'Mod 1',' Mod 2', 'Joint'})
title('NMI between estimated parcellation and ground truth parcellation')
